#  +-------------------------------------------------------------------------
#  | Copyright (C) 2016 Yunify, Inc.
#  +-------------------------------------------------------------------------
#  | Licensed under the Apache License, Version 2.0 (the "License");
#  | you may not use this work except in compliance with the License.
#  | You may obtain a copy of the License in the LICENSE file, or at:
#  |
#  | http://www.apache.org/licenses/LICENSE-2.0
#  |
#  | Unless required by applicable law or agreed to in writing, software
#  | distributed under the License is distributed on an "AS IS" BASIS,
#  | WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  | See the License for the specific language governing permissions and
#  | limitations under the License.
#  +-------------------------------------------------------------------------

require 'active_support/core_ext/hash/keys'

module QingCloud
  module SDK
    class JobService
      attr_accessor :config, :properties

      def initialize(config, properties)
        self.config     = config
        self.properties = properties.deep_symbolize_keys
      end

      # Documentation URL: https://docs.qingcloud.com/api/job/describe_jobs.html
      def describe_jobs(jobs: [], limit: nil, offset: nil, status: [], verbose: nil)
        input = {
          config:         config,
          properties:     properties,
          api_name:       'DescribeJobs',
          request_method: 'GET',
          request_params: {
            'jobs'    => jobs,
            'limit'   => limit,
            'offset'  => offset,
            'status'  => status,
            'verbose' => verbose, # verbose's available values: 0
          },
        }

        describe_jobs_input_validate input

        request = Request.new input
        request.send
      end

      private

      def describe_jobs_input_validate(input)
        input.deep_stringify_keys!

        if input['request_params']['verbose'] && !input['request_params']['verbose'].to_s.empty?
          verbose_valid_values = ['0']
          unless verbose_valid_values.include? input['request_params']['verbose'].to_s
            raise ParameterValueNotAllowedError.new(
              'verbose',
              input['request_params']['verbose'],
              verbose_valid_values,
            )
          end
        end
      end

      public
    end
  end
end
